"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.imageEnricherGitInfo = void 0;
const bluebird_1 = __importDefault(require("bluebird"));
const lodash_1 = __importDefault(require("lodash"));
const chalk_1 = __importDefault(require("chalk"));
const types_1 = require("./types");
const git_providers_1 = require("../api/git-providers");
const codefresh_1 = require("../api/codefresh");
async function resolvePullRequests(inputs, provider) {
    if (inputs.eventFilePR) {
        return bluebird_1.default.each(inputs.eventFilePR, async (pr) => {
            const commitsInfo = await provider.getCommitsInfo(inputs.repo, pr.number);
            return {
                ...pr,
                ...commitsInfo,
            };
        });
    }
    return provider.getPullRequestsWithCommits(inputs.repo, inputs.branch);
}
async function patchImageWithGitBranchData(codefreshAPI, provider, pullRequests, inputs) {
    if (lodash_1.default.isEmpty(pullRequests)) {
        console.warn(`no PRs have been found, looking for the branch data`);
        const branch = await provider.getBranch(inputs.repo, inputs.branch);
        if (branch) {
            await codefreshAPI.patchImageWithGitBranchData(inputs.imageName, branch);
            console.log(`image patched`);
        }
        else {
            console.warn(`branch "${inputs.branch}" not found, ensure that it is not deleted`);
        }
    }
    else {
        console.log(chalk_1.default.green(`retrieved prs ${JSON.stringify(pullRequests, null, 2)}`));
        const lastCommit = git_providers_1.GitProviders.getTheMostRecentCommit(pullRequests);
        await codefreshAPI.patchImageWithGitBranchData(inputs.imageName, {
            name: inputs.branch,
            commit: lastCommit.sha,
            commitMsg: lastCommit.message,
            commitURL: lastCommit.url,
            committerUsername: lastCommit.userName
        });
        console.log(`image patched`);
    }
}
async function imageEnricherGitInfo(inputs) {
    console.log('starting git enricher library');
    if (inputs.provider === git_providers_1.providers.GITHUB && inputs.gitContext && !inputs.githubContextName) {
        inputs.githubContextName = inputs.gitContext;
    }
    const { cfHost, cfApiKey } = inputs;
    const codefreshAPI = new codefresh_1.CodefreshAPI(cfHost, cfApiKey);
    const Provider = git_providers_1.GitProviders.get(inputs.provider);
    const provider = new Provider(inputs);
    const pullRequests = await resolvePullRequests(inputs, provider)
        .catch((error) => console.error(`Can't get pull requests. Error: ${error.message}`));
    if (inputs.platform === types_1.Platform.GITOPS) {
        await patchImageWithGitBranchData(codefreshAPI, provider, pullRequests, inputs);
    }
    if (lodash_1.default.isEmpty(pullRequests)) {
        console.warn(`pullRequests array is empty`);
        return;
    }
    const gitopsImageReportingFeature = await codefreshAPI.getFeature('gitopsImageReporting');
    for (const pr of pullRequests) {
        if (inputs.platform === types_1.Platform.CLASSIC) {
            try {
                console.log(`creating classic annotation for image ${inputs.imageName}`);
                await codefreshAPI.createPullRequestAnnotationClassic(inputs.imageName, pr);
                console.log(`classic annotation created`);
            }
            catch (error) {
                console.error(`failed to assign pull request ${pr.number} to your classic image ${inputs.imageName}, reason: ${error.message}`);
            }
        }
        if (inputs.platform === types_1.Platform.GITOPS || gitopsImageReportingFeature) {
            try {
                console.log(`creating gitops annotation for image ${inputs.imageName}`);
                await codefreshAPI.createPullRequestAnnotation(inputs.imageName, pr);
                console.log(`gitops annotation created`);
            }
            catch (error) {
                console.error(`failed to assign pull request ${pr.number} to your gitops image ${inputs.imageName}, reason: ${error.message}`);
            }
        }
    }
}
exports.imageEnricherGitInfo = imageEnricherGitInfo;
//# sourceMappingURL=index.js.map