import _ from 'lodash'
import { Version2Client } from 'jira.js'

import { CodefreshAPI } from '../codefresh'
import { JiraApi } from './jira.api'
import { JiraEnricherInputs } from '../../image-enricher-jira-info/types'

export class JiraService {
    private inputs
    private jiraConfig
    private jira
    private readonly codefreshAPI

    constructor(inputs: JiraEnricherInputs) {
        this.inputs = inputs
        this.jiraConfig = _.cloneDeep(inputs.jira)
        this.codefreshAPI = new CodefreshAPI(inputs.cfHost, inputs.cfApiKey)
    }

    async init() {
        if (this.jiraConfig.context) {
            console.log('using jira context auth')
            const jiraContext = await this.codefreshAPI.getContext(this.jiraConfig.context, 'atlassian')
            if (!jiraContext) {
                throw new Error(`Codefresh jira integration "configuration.jira.context" not found`)
            }
            if (jiraContext.spec.data.auth.type === 'addon') {
                this.jira = new JiraApi(this.jiraConfig.context, this.codefreshAPI)
                return
            }

            this.jiraConfig = {
                host: jiraContext.spec.data.auth.apiURL,
                authentication: {
                    basic: {
                        email: jiraContext.spec.data.auth.username,
                        apiToken: jiraContext.spec.data.auth.password
                    },
                },
                context: this.jiraConfig.context,
            }
        }

        if (this.jiraConfig.authentication.basic) {
            console.log('using jira basic auth')
        } else if (this.jiraConfig.authentication.personalAccessToken) {
            console.log('using jira personalAccessToken auth')
        } else {
            throw new Error('Jira credentials not provided')
        }

        this.jira = new Version2Client({
            ...this.jiraConfig
        })

    }

    extract() {
        return new RegExp(`${this.inputs.projectName}-\\d*`, 'i').exec(this.inputs.message)
    }

    async getInfoAboutIssue(issue) {
        try {
            return await this.jira.issues.getIssue({
                issueIdOrKey: issue,
            })
        } catch (error) {
            return this._handleJiraError(error, issue)
        }
    }

    _handleJiraError(error, issueId) {
        const errorMessage = JSON.stringify(error.response.data)
        console.error(`failed to get jira issue ${issueId}: ${errorMessage}`)

        if (error.response.status === 401) {
            throw new Error('failed to authenticate to Jira, please verify you are using valid credentials')
        }
        if (error.response.status === 404) {
            if (this.inputs.failOnNotFound === 'true') {
                throw new Error(`issue ${issueId} not found`)
            } else {
                console.warn(`skip issue ${issueId}, didn't find in jira system or you don't have permissions for find it`)
                return null
            }
        }
    }
}
